#!/bin/bash
#
#This module optimize sh, lo, re, mi, and ma parameters for daospec
#and stores results in the file defined by the variable file_opt_m1.
#This file is organized in columns:
#column_1    -> complete spectra path name
#column_2-17 -> daospec option parameters (or sh lo le ri re mi ma ve fi sc cr wa sm ba fw)
#column_18   -> flag1 for fix (flag1=1) or not fix (flag1=0) fwhm analysis
#column_19   -> U for UVES spectra, G for GIRAFFE
#For known errors (DAOSPEC or IRAF output files not present or spectrum header keyword
#BITPIX different from 32) it reports a error message which starts with '#'
#
#INFORMATION/FILES NEEDED:
#a - WORKING DIRECTORY AND STUFF DIRECTORY MUST BE DEFINED IN THE FILE pipe.opt
#    WORKING DIRECTORY: is the directory where you have this script and the input file for this 
#    module (see below)
#    STUFF DIRECTORY: is the directory which contains the sub-script and needed files (see below)
#b - IT NEEDS THE FOLLOWING FILES:
#    login.cl		YOUR IRAF login.cl (PUT IT INSIDE STUFF DIRECTORY)
#    shlobpix.cl	IRAF script to compute sh and lo DAOSPEC parameters (PUT IT INSIDE SCRIPT DIRECTORY)
#    re.cl		IRAF script to compute residual core flux DAOSPEC parameter (PUT IT INSIDE SCRIPT DIRECTORY)
#    laboratory.dat	linelist for DAOSPEC
#
#INPUT FILE: A file with a list of the spectra file names (with the complete
#directory tree) is needed and the name (inputlist.dat default) can be
#specified in pipe.opt.
#This file can be written by hand or using the following command
#find $WORK_DIR -name \*l.fits -not -name \*uall\* | grep -v EXP > inputlist.dat
#and should contain three columns:
#column_1: complete_path_name_of_each_spectra_you_want_to_analyse
#column_2: input continuum order
#column_3: input fwhm
#column_4: 1 or 0 to say if you want FIX FWHM (1) or you don't (0)
#column_5: input radial velocity if known otherwise 0
#column_6: input residual core flux if known otherwise 0
#column_7: U for UVES, G for GIRAFFE
#
#OUTPUT FILES: for this module are the DAOSPEC files (contained in the directories
#of each spectrum) and outfile defined in pipe.opt (output01.dat default)

echo M01: ----------   Module 01 is now running...   ----------

infile=`grep -w inputf01 pipe.opt | cut -d= -f2`	#input file
outfile=`grep -w output01 pipe.opt | cut -d= -f2`	#Output filename
majrevfile=`grep MAJfile.*= pipe.opt | cut -d= -f2`  #Major reviews file

#CHECKING NEEDED FILE before going on
if [ -z $outfile ]; then
  echo M01: output file is not defined in pipe.opt ...
  outfile=output01.dat
  echo ...setting it as $outfile.
fi
file_opt_m1=${WORK_DIR}/${outfile}			#Output file of this module
echo "#" >> $file_opt_m1

if [ -z $infile ]; then
  echo M01: output file is not defined in pipe.opt ...
  infile=inputlist.dat
  echo ...setting it as $infile.
fi
infile=${WORK_DIR}/$infile

if [ -z $majrevfile ]; then
  echo M01: major reviews file is not defined in pipe.opt ...
  majrevfile=majorrev.dat
  echo ...setting it as $majrevfile.
fi
majrevfile=${WORK_DIR}/$majrevfile
rm -f $majrevfile

rusco=`grep -v \# $infile | wc -l`
if [ $rusco -eq 0 -o -z $rusco -o ! -f $infile ]; then 
  echo M01: $infile has no lines or does not exist. Exiting the module.
  exit 1
fi
echo M01: file $infile has $rusco spectra. 

list=(`grep -v \# $infile | awk '{print $1}'`)
ord=(`grep -v \# $infile | awk '{print $2}'`)
fwhmi=(`grep -v \# $infile | awk '{print $3}'`)
fix=(`grep -v \# $infile | awk '{print $4}'`)
vrin=(`grep -v \# $infile | awk '{print $5}'`)
rcf=(`grep -v \# $infile | awk '{print $6}'`)
stype=(`grep -v \# $infile | awk '{print $7}'`)

for i in ${#list[@]} ${#ord[@]} ${#fwhmi[@]} ${#fix[@]} ${#vrin[@]} ${#rcf[@]} ${#stype[@]}; do
  if [ $i -eq 0 -o $i != $rusco ]; then
    echo -e "M01: cannot find needed information in $infile. Is it in the correct format?"
    echo -e "M01: (eg. spectrum ord fwhm fix RV rcf spectrum_type).\nM01: Exiting the module."
    exit 1
  fi
done  

#CHECKING DUPLICATES IN inupt file and spectra in common with output file
checkduplicate=`grep -v \# $infile | awk 'x[$1]++' | wc -l`
if [ $checkduplicate -gt 0 ]; then echo M01: file $infile contains duplicated spectra. Please correct it\!; exit 1; fi
if [ -f $file_opt_m1 ]; then
  checkduplicate=`cat $infile $file_opt_m1 | grep -v \# | awk 'x[$1]++' | wc -l` 
  if [ $checkduplicate -gt 0 ]; then 
    echo M01: file $infile and $file_opt_m1 have spectra in common. 
    echo M01: This may crash the pipeline. Please correct it.; exit 1;
  fi
fi

echo M01: Starting automatic procedure on spectra...
trap 'echo M01: ...forced to exit.; exit 1' 2	#this command listen to the signal CTLR+C and force the script to stop with exit status 1
index=0						#variable used inside the for loop
notd=0						#variable used to count spectra with no daospec output!
errs=0						#variable used to count spectra which encountered known errors during the analysis
for fits in ${list[@]}; do
  skip=0					#variable used to skip a file
  #Creating two separate arrays containing standard DAOSPEC option parameters
  opt=(or=30 sh=4820. lo=5780. le=5000. ri=5050. re=18. mi=-500 ma=500 ve=3.0 fi=1 sc=0 cr=1 wa=0 sm=5 ba=0.0001 fw=9)

  echo M01: ...playing with file $fits...
  echo $fits >> $majrevfile
  path=`echo $fits | sed -e "s/\/[^\/]*$//"`
  file=`echo $fits | sed -e "s/.*\///"`
  rootname=`echo $file | cut -d. -f1`
  #CHECKING IF FILES in the input file do exist
  if [ ! -f $fits ]; then echo M01: File $fits does not exist. Skipping to the next spectrum.; 
      echo "#ERROR: $fits does not exist" >> $file_opt_m1
      ((errs++))
      skip=1
  fi
  if [ $skip -eq 0 ]; then 
    cd $path			#change directory  
    #Getting the number of spectrum header lines using awk 
    line=`awk '{where=match($0," END ");if(where){printf "%.0f",where/80}}' $file`
    #Extracting the information about the format of the spectrum
    bitpix=`fold $file | head -${line} | grep BITPIX | awk '{ print ($3>=0) ? $3:0-$3}'` 
      
    if [ $bitpix != 32 ]; then
      echo "M01: File $fits is not in 32 bit format. Skipping to the next spectrum."
      echo "#ERROR: $fits is not in 32 bit format" >> $file_opt_m1
      sed -i.bkp 's|'$fits'|'$fits'\'$'\n\t*M01: not in 32 bit format|' $majrevfile
      ((errs++))
    else 
      #removing files
      rm -f shlobpix.cl re.cl login.cl laboratory.dat
      #Copying file to the directory of the spectrum    
      cp $STUFF_DIR/shlobpix.cl .  
      cp $STUFF_DIR/re.cl .    
      cp $STUFF_DIR/login.cl . 
      cp $STUFF_DIR/laboratory.dat .
    
      #RUNNING IRAF to estimate sh and lo (wavelength limits)
      #removing possible previous output of the IRAF script
      rm -f output_M01.iraf
      echo $fits > input.iraf
      echo -e "\nM01: Now running IRAF to get sh and lo DAOSPEC parameters..."
      cl < shlobpix.cl -o
    
      if [ ! -f output_M01.iraf ]; then 
        echo -e "\nM01: Problem with IRAF (shlobpix.cl script) and file $fits. Skipping to the next spectrum."
        echo "#ERROR: $fits  problem with shlobpix.cl IRAF sript." >> $file_opt_m1
        sed -i.bkp 's|'$fits'|'$fits'\'$'\n\tM01: problem with shlobpix.cl IRAF sript|' $majrevfile
        ((errs++))
        skip=1
      else
        null=`wc -l < output_M01.iraf`
	if [ $null != 2 ]; then 
          echo -e "\nM01: Problem with IRAF (shlobpix.cl script) and file $fits. Skipping to the next spectrum."
          echo "#ERROR: $fits  problem with shlobpix.cl IRAF sript." >> $file_opt_m1
          sed -i.bkp 's|'$fits'|'$fits'\'$'\n\tM01: problem with shlobpix.cl IRAF sript|' $majrevfile
          ((errs++))
          skip=1 
	fi
      fi  
      if [ $skip -eq 0 ]; then
        echo -e "\n...done!"
        lambdastart=`grep sh= output_M01.iraf | cut -d= -f2`
        lambdastart=$( printf "%.0f" $lambdastart)		#print the integer part
  	lambdastop=`grep lo= output_M01.iraf | cut -d= -f2`
	lambdastop=$(($( printf "%.0f" $lambdastop)+1))	  
	opt[0]=or=${ord[$index]}
        opt[1]=`grep sh= output_M01.iraf`
        opt[2]=`grep lo= output_M01.iraf`
        opt[9]=fi=${fix[$index]}
        opt[15]=fw=${fwhmi[$index]}   #adding fw parameter to opt array  
	
	if [ ${vrin[$index]} != 0 ]; then			#If radial velocity is known it sets it	 
	  echo "M01: Information about radial velocity found in $infile for $file."   
          rusco=`awk -v PIP="${vrin[$index]}" 'BEGIN {print PIP-10}'`
          opt[6]=mi=$rusco
          rusco=`awk -v PIP="${vrin[$index]}" 'BEGIN {print PIP+10}'`
          opt[7]=ma=$rusco
	  #If residual core flux is not known and radial velocity is, it calculates it
	  if [ ${rcf[$index]} == 0 ]; then			
	    #The iraf script re.cl looks for Halfa and Hbeta. If a spectrum
	    #has another wavelength range the pipeline skips it
            if [ $lambdastart -gt 4000 -a $lambdastop -le 10100 ]; then
	      vrad=${vrin[$index]}
              #SECOND RUN of IRAF to estimate residual core flux
              echo $fits  $vrad > input.iraf
              echo -e "\nM01: Now running IRAF to get residual core flux DAOSPEC parameter..."
              cl < re.cl -o	      
    
              if [ ! -f output_M01.iraf ]; then 
        	echo -e "\nM01: Problem with IRAF (re.cl script) and file $fits. Skipping to the next spectrum."
        	echo "#ERROR: $fits  problem with re.cl IRAF sript. The parameter found till now are ${opt[0]} ${opt[1]} ${opt[2]} ${opt[6]} ${opt[7]} ${opt[9]} ${opt[15]}" >> $file_opt_m1
                sed -i.bkp 's|'$fits'|'$fits'\'$'\n\tM01: problem with re.cl IRAF sript|' $majrevfile
                ((errs++))
        	skip=1
      	      else
        	null=`wc -l < output_M01.iraf`
		if [ $null != 3 ]; then 
          	  echo -e "\nM01: Problem with IRAF (re.cl script) and file $fits. Skipping to the next spectrum."
          	  echo "#ERROR: $fits  problem with re.cl IRAF sript. The parameter found till now are ${opt[0]} ${opt[1]} ${opt[2]} ${opt[6]} ${opt[7]} ${opt[9]} ${opt[15]}" >> $file_opt_m1
                  sed -i.bkp 's|'$fits'|'$fits'\'$'\n\tM01: problem with re.cl IRAF sript|' $majrevfile
          	  ((errs++))
		  skip=1 
		else
	          echo -e "\n...done!"
                  opt[5]=`grep re= output_M01.iraf`
		  _rc=`grep re= output_M01.iraf | cut -d= -f2`
		  if [ $_rc -eq 18 ]; then
		    sed -i.bkp 's|'$fits'|'$fits'\'$'\n\t*M01: residual core flux set to 18|' $majrevfile
		  fi
		fi
      	      fi 
	    else
	      echo -e "M01: $fits spectral range out of pipeline bounds. Skipping to the next spectrum."
	      echo "#ERROR: $fits spectral range out of pipeline bounds. The parameter found till now are ${opt[0]} ${opt[1]} ${opt[2]} ${opt[6]} ${opt[7]} ${opt[9]} ${opt[15]}" >> $file_opt_m1
	      sed -i.bkp 's|'$fits'|'$fits'\'$'\n\tM01: spectral range out of pipeline bounds|' $majrevfile
              ((errs++))
	      skip=1
	    fi
	  fi
	  string="M01: Now running DAOSPEC..."
	else
	  string="M01: No information about radial velocity in $infile for $file. \nM01: Now running DAOSPEC to get radial velocity..."
	fi
	if [ $skip -eq 0 ]; then
	  if [ ${rcf[$index]} != 0 ]; then
	    echo "M01: Information about residual core flux found in $infile for $file."
	    opt[5]=re=${rcf[$index]}
	  fi 
  
          #FIRST RUN of DAOSPEC to estimate Radial Velocity 
          #remove daospec files
          rm -f daospec.opt ${rootname}.daospec ${rootname}C.fits ${rootname}R.fits		  
          for optc in ${opt[@]}; do	  		#create daospec.opt file using opt vector
            echo -e "$optc" >> daospec.opt
          done
	  
	  echo -e "\n$string"
	  echo "#DAOSPEC module 01 logfile" > ${rootname}_M01.log
	  cat daospec.opt >> ${rootname}_M01.log
daospec << END >> ${rootname}_M01.log			#run daospec and save output in a logfile
    
$file
  
END

          vrad=`head -1 ${rootname}.daospec | awk '{print $4}'`
	  okv=0
	  okf=0
	  if [[ "$vrad" =~ ^[-0-9]+([.][0-9]+)?$ ]] ; then
	    #If radial velocity is defined in the DAOSPEC output file
	    #then it updates the parameters that will be printed in 
	    #the output file file_opt_m1	    
            rusco=`awk -v PIP="$vrad" 'BEGIN {print PIP-10}'`
            opt[6]=mi=$rusco
            rusco=`awk -v PIP="$vrad" 'BEGIN {print PIP+10}'`
            opt[7]=ma=$rusco
	    okv=1
	    if [ ${vrin[$index]} != 0 ]; then
	      sed -i.bkp 's|'$fits'|'$fits'\'$'\n\tM01: RV changed from '${vrin[$index]}' to '$vrad'|' $majrevfile
	    else
	      sed -i.bkp 's|'$fits'|'$fits'\'$'\n\tM01: RV masurement is '$vrad'.|' $majrevfile
	    fi	        
	  else
	    echo "M01: Problem with DAOSPEC and $fits. Radial velocity cannot be defined."
	    sed -i.bkp 's|'$fits'|'$fits'\'$'\n\tM01: cannot define RV using DAOSPEC|' $majrevfile              
	  fi	    
	  rusco=`head -1 ${rootname}.daospec | awk '{print $10}' | cut -d, -f1`
	  if [[ "$rusco" =~ ^[-0-9]+([.][0-9]+)?$ ]] ; then
	    #If fwhm is defined in the DAOSPEC output file
	    #then it updates the parameter that will be printed in 
	    #the output file file_opt_m1
	    opt[15]=fw=$rusco
	    echo "M01: ...done!"
	    okf=1
	    _temp=`awk -v PIP="$rusco" -v PLU="${fwhmi[$index]}" 'function abs(x){return ((x < 0.0) ? -x : x)} BEGIN{FRAC=100*(PIP-PLU)/PLU;{printf "%i",abs(FRAC)}}'`
	    if [ $_temp -gt 70 ]; then
	      sed -i.bkp 's|'$fits'|'$fits'\'$'\n\t*M01: FWHM largely (more than 70%) changed from input value passing from '${fwhmi[$index]}' to '$rusco'|' $majrevfile
	    else
	      sed -i.bkp 's|'$fits'|'$fits'\'$'\n\tM01: FWHM changed from '${fwhmi[$index]}' to '$rusco'|' $majrevfile
	    fi	      
	  else
	    echo "M01: Problem with DAOSPEC and $fits. Output FWHM cannot be get from ${rootname}.daospec."
	    sed -i.bkp 's|'$fits'|'$fits'\'$'\n\tM01: cannot define FWHM using DAOSPEC|' $majrevfile	    
	  fi            
	  
	  if [ ! -s ${rootname}.daospec -o $okf -eq 0 -o $okv -eq 0 ]; then
              echo "M01: Skipping to the next spectrum."
	      echo "M01: removing DAOSPEC output file:"
	      rm -fv ${rootname}.daospec 
              echo "#ERROR: $fits problem with DAOSPEC (radial velocity and output FWHM cannot be defined). The parameter found till now are  ${opt[0]} ${opt[1]} ${opt[2]} ${opt[9]}" >> $file_opt_m1
	      ((errs++))
	      ((notd++))
	  else	      
	    #if residual core flux and radial velocity are not known since the beginning 
	    #then it calculates residual core flux using radial velocity derived with DAOSPEC
	    if [ ${rcf[$index]} == 0 -a ${vrin[$index]} == 0 ]; then	      
	      #The iraf script re.cl looks for Halfa and Hbeta. If a spectrum
	      #has another wavelength range the pipeline skips it	
              if [ $lambdastart -gt 4000 -a $lambdastop -le 10100 ]; then
                #SECOND RUN of IRAF to estimate residual core flux
                echo $fits  $vrad > input.iraf
                echo -e "\nM01: Now running IRAF to get residual core flux DAOSPEC parameter..."
                cl < re.cl -o		
		
                if [ ! -f output_M01.iraf ]; then 
        	  echo -e "\nM01: Problem with IRAF (re.cl script) and file $fits. Skipping to the next spectrum."
        	  echo "#ERROR: $fits  problem with re.cl IRAF sript. The parameter found till now are  ${opt[0]} ${opt[1]} ${opt[2]} ${opt[6]} ${opt[7]} ${opt[9]} ${opt[15]}" >> $file_opt_m1
      	      	  sed -i.bkp 's|'$fits'|'$fits'\'$'\n\tM01: problem with re.cl IRAF sript|' $majrevfile
          	  ((errs++))
		else
        	  null=`wc -l < output_M01.iraf`
		  if [ $null != 3 ]; then 
          	    echo -e "\nM01: Problem with IRAF (re.cl script) and file $fits. Skipping to the next spectrum."
          	    echo "#ERROR: $fits  problem with re.cl IRAF sript.  The parameter found till now are  ${opt[0]} ${opt[1]} ${opt[2]} ${opt[6]} ${opt[7]} ${opt[9]} ${opt[15]}" >> $file_opt_m1
          	    sed -i.bkp 's|'$fits'|'$fits'\'$'\n\tM01: problem with re.cl IRAF sript|' $majrevfile
          	    ((errs++))
		    skip=1 
		  else
	            echo -e "\n...done!"
                    opt[5]=`grep re= output_M01.iraf`
		    _rc=`grep re= output_M01.iraf | cut -d= -f2`
		    if [ $_rc -eq 18 -o $_rc -eq 5 ]; then
		      sed -i.bkp 's|'$fits'|'$fits'\'$'\n\t*M01: residual core flux set to standard value (5 or 18)|' $majrevfile
		    fi
                    echo $fits ${opt[@]} ${fix[$index]} ${stype[$index]} >> $file_opt_m1
		  fi
      	      	fi	      
	      else
	        echo -e "M01: $fits spectral range out of pipeline bounds. Skipping to the next spectrum."
	        echo "#ERROR: $fits spectral range out of pipeline bounds. The parameter found till now are  ${opt[0]} ${opt[1]} ${opt[2]} ${opt[6]} ${opt[7]} ${opt[9]} ${opt[15]}" >> $file_opt_m1
	        sed -i.bkp 's|'$fits'|'$fits'\'$'\n\tM01: spectral range out of pipeline bounds|' $majrevfile
                ((errs++))
	      fi
	    else
              echo $fits ${opt[@]} ${fix[$index]} ${stype[$index]} >> $file_opt_m1
	    fi	  	  
	  fi
        fi
      fi
      #After having performed computations it clears the fits directory from not needed files
      rm -f ${path}/shlobpix.cl 
      rm -f ${path}/re.cl 
      rm -f ${path}/login.cl 
      rm -f ${path}/input.iraf 
      rm -f ${path}/output_M01.iraf    
    fi      
  fi
  ((index++))
done
cd $WORK_DIR
echo -e "\nM01: Module 01 analysed ${#list[@]} spectra. $errs spectra encountered some problems during the anlysis. $notd spectra of $errs do not have DAOSPEC output." 
echo -e "\nM01: Module 01 has ended its duty!"

exit 0
   
