#!/bin/bash
#
#module 4
#
#This module computes DAOSPEC normalized (*N.fits) and fit spectra (*F.fits) 
#for each spectrum that is listed inside the input file (see below). 
#Then, using the average of the residuals of the DAOSPEC fit R% (information 
#in the header of the DAOSPEC output file), it runs DAOSPEC on the normalized 
#spectrum multiplied by 1+R% ("upper" spectrum -> *NU.fits) and on the 
#normalized spectrum multiplied by 1-R% ("lower" spectrum -> *NL.fits) using 
#parameter or=-1.
#Afterwards it computes the EW difference between the "best" and the "upper" 
#set of measurements, and between the "best" and the "lower" set of measurements. 
#This differences are print in the last two column of the output file.
#There are two OUTPUT FILES: 
#one for each spectrum in their dyrectory (*M04.daospec), 
#containing the same information of the DAOSPEC output file (*.daospec)
#but for only the matched lines plus two column containing, in order, the
#differences for the EWs;
#one containing information about the final status of the module 
#defined in pipe.opt (output04.dat default)
#
#INFORMATION/FILES NEEDED:
#a - WORKING DIRECTORY AND STUFF DIRECTORY MUST BE DEFINED IN THE FILE pipe.opt
#    WORKING DIRECTORY: is the directory where you have this script and the input file for this 
#    module (see below)
#    STUFF DIRECTORY: is the directory which contains the sub-script and needed files (see below)
#b - IT NEEDS THE FOLLOWING FILES:
#    login.cl		YOUR IRAF login.cl (PUT IT INSIDE STUFF DIRECTORY)
#    arith.cl		IRAF script to compute DAOSPEC normalized and fit spectra
#    laboratory.dat	linelist for DAOSPEC
#    DAOSPEC files	the DAOSPEC output files obtained after having run module 3
#
#INPUT FILE: A file with a list of the spectra file names (with the complete
#directory tree) is needed followed by the DAOSPEC parameters in the following
#MANDATORY order:
#column_1 -> complete_path_name_of_each_spectra_you_want_to_analyse
#column_2-17 -> daospec option parameters (or sh lo le ri re mi ma ve fi sc cr wa sm ba fw)
#
#OUTPUT FILES:for this module are the DAOSPEC files (contained in the directories
#of each spectrum), the IRAF files, the outfile defined in pipe.opt (output04.daospec default),
#and the results of the module written in path_of_the_spectrum/*M04.daospec and:
#	*N.fits		-> normalized spectrum
#	*F.fits		-> fit spectrum
#

echo M04: ----------   module 04 is now running...   ----------


#CHECKING NEEDED FILE before going on
infile=`grep -w output03 pipe.opt | cut -d= -f2`
majrevfile=`grep MAJfile.*= pipe.opt | cut -d= -f2`     #Major reviews file

if [ -z $infile ]; then
  echo "M04: input file (module 03 output) is not defined in pipe.opt ..."
  echo "M04: ...setting it as output03.dat"
  infile=output03.dat
fi
file_opt_m3=${WORK_DIR}/${infile}

if [ ! -f $file_opt_m3 ]; then
  echo M04: I cannot find the output file of module 3 $file_opt_m3.
  echo M04: Exiting the module.
  exit 1
fi

outfile=`grep -w output04 pipe.opt | cut -d= -f2`
if [ -z $outfile ]; then
  echo M04: output file is not defined in pipe.opt ...
  echo M04: ...setting it as output04.dat
  outfile=output04.dat
fi
out05=${WORK_DIR}/${outfile}
rm -f $out05


if [ -z $majrevfile ]; then
  echo M04: major reviews file is not defined in pipe.opt ...
  majrevfile=majorrev.dat
  echo ...setting it as $majrevfile.
fi
majrevfile=${WORK_DIR}/$majrevfile

list=(`grep -v \# $file_opt_m3 | awk '{print $1}'`)
ord=(`grep -v \# $file_opt_m3 | awk '{print $2}'`)
fix=(`grep -v \# $file_opt_m3 | awk '{print $11}'`)
fwhmi=(`grep -v \# $file_opt_m3 | awk '{print $17}'`)
#flag3=(`grep -v \# $file_opt_m2 | awk '{print $20}'`)

if [ ${#list[@]} -eq 0 ]; then
  echo M04: No usable spectra in $file_opt_m3. Exiting the module.
  exit 1
fi


trap 'echo M04: ...forced to exit.; exit 1' 2	#this command listen to the signal CTLR+C and force the script to stop with exit status 1
index=0						#variable used inside the for loop
notd=0						#variable used to count spectra with no daospec output!
errs=0						#variable used to count spectra which encountered known errors during the analysis
for fits in ${list[@]}; do						
  skip=0					#variable used to skip file			
  echo M04: ...playing with file $fits...
  path=`echo $fits | sed -e "s/\/[^\/]*$//"`
  file=`echo $fits | sed -e "s/.*\///"`
  rootname=`echo $file | cut -d. -f1`
  opts=(`grep $fits $file_opt_m3 | awk '{print $2"  "$3"  "$4"  "$5"  "$6"  "$7"  "$8"  "$9"  "$10"  "$11"  "$12"  "$13"  "$14"  "$15"  "$16"  "$17}'`)
  cd $path			#change directory
  #check for file
  if [ ! -f $fits -o ! ${rootname}.daospec -o ! ${rootname}C.fits ]; then 
    echo M04: Cannot find DAOSPEC output for $fits. Skipping to the next spectrum.
    echo "#ERROR: $fits or ${rootname}.daospec or ${rootname}C.fits do not exist." >> $out05
    skip=1
    sed -i.bkp 's|'$fits'|'$fits'\'$'\n\t*M04: Cannot find DAOSPEC output files (a).|' $majrevfile
  fi
  if [ $skip -eq 0 ]; then
    #removing files
    rm -f arith.cl login.cl laboratory.dat
    #Copying file to the directory of the spectrum     
    cp $STUFF_DIR/arith.cl .     
    cp $STUFF_DIR/login.cl .     
    cp $STUFF_DIR/laboratory.dat .
    
    #here it extracts the average of the residuals spectra from the *.daospec header
    resid=`head -2 ${rootname}.daospec | grep "Relative flux dispersion in residual spectrum" | awk '{print $8}' | sed 's/%*$//'`  #the sed command replace the % with a space
    rusco=`echo ${ord[$index]} | cut -d= -f2`
    
    #checking if resid is a real and if rusco is an integer otherwise exit
    if ! [[ "$resid" =~ ^[-0-9]+([.][0-9]+)?$ && "$rusco" =~ ^[-0-9]+$ ]] ; then
      echo M04: Cannot find information about the average of the residuals in ${rootname}.daospec. Skipping to the next spectrum.
      echo "#ERROR: $fits needed DAOSPEC information not found." >> $out05
      ((errs++))
      skip=1
      sed -i.bkp 's|'$fits'|'$fits'\'$'\n\t*M04: Cannot find DAOSPEC output files (b).|' $majrevfile
    fi
    if [ $skip -eq 0 ]; then 
      #RUNNING IRAF to compute normalized spectrum, normalized*(1+R%), and normalized*(1-R%)
      #removing possible previous output of the IRAF script
      rm -f output_M04.iraf ${rootname}N.fits ${rootname}NU.fits ${rootname}NL.fits ${rootname}F.fits
      echo M04: Now I am going to run IRAF ...
      echo $fits  ${rootname}  $resid  $rusco > input.iraf
      cl < arith.cl -o
    
      #checking if IRAF has done well
      if [ ! -f ${rootname}NU.fits -o ! -f ${rootname}NL.fits -o ! -f ${rootname}F.fits ]; then 
	  echo -e "\nM04: Problem with IRAF script arith.cl and file $fits. Skipping to the next spectrum."
	  echo "#ERROR: $fits  problem with IRAF." >> $out05
	  ((errs++))
	  skip=1
          sed -i.bkp 's|'$fits'|'$fits'\'$'\n\t*M04: Problem with IRAF script arith.cl.|' $majrevfile
      fi
      if [ $skip -eq 0 ]; then
        echo -e "\n...done!"
        #RUNNING DAOSPEC
        #removing daospec files
        rm -f daospec.opt ${rootname}NU.daospec ${rootname}NL.daospec
        rm -f ${rootname}NUC.fits ${rootname}NLC.fits
        rm -f ${rootname}NUR.fits ${rootname}NLR.fits
        #creating daospec.opt with or=-1
        opts[0]=or=-1
	opts[9]=fi=1
        for opt in ${opts[@]}; do	
          echo -e "$opt" >> daospec.opt
        done
        
	echo -e "\nM04: file ${rootname}N.fits (if or>0) ${rootname}NU.fits ${rootname}NL.fits ${rootname}F.fits have been created."
        echo -e "\nM04: Now running DAOSPEC on ${rootname}NU.fits ${rootname}NL.fits..."	  
        echo "#DAOSPEC module 04 logfile" > ${rootname}_M04.log
        cat daospec.opt >> ${rootname}_M04.log
daospec << END >> ${rootname}_M04.log	#run daospec and save output in a logfile
  

${rootname}NU.fits
${rootname}NL.fits
  
END

        #computing the differences between the "best" and "upper" sets of 
        #EW measurements and between the "best" and "lower" for only the 
        #matched lines of the linelist. Then it prints in the outputfile
        #the DAOSPEC results for the "best" set followed by the
        #two differences 
        if [ ! -s ${rootname}.daospec -o ! -s ${rootname}NU.daospec -o ! -s ${rootname}NL.daospec ]; then 
           echo -e "\nM04: Problem with DAOSPEC and $fits. Skipping to the next spectrum."
	   echo "#ERROR: $fits  problem with DAOSPEC." >> $out05 
	   ((errs++))
	   ((notd++))       
           skip=1
           sed -i.bkp 's|'$fits'|'$fits'\'$'\n\t*M04: Cannot find DAOSPEC output files (c).|' $majrevfile
        fi 
	#FINALLY PRINTING RESULTS on OUTPUTFILE *M04.daospec
        if [ $skip -eq 0 ]; then
          header=`head -2 ${rootname}.daospec`
          echo -e "\n...done!"
	  outfM04=${path}/${rootname}M04.daospec
	  echo \# $header > $outfM04
          for dummy in `awk '!/^#/ {print $1}' laboratory.dat`; do
     	    ewNU=`grep ${dummy}.*+0.00 ${rootname}NU.daospec | awk '{print $3}'`
     	    ewNL=`grep ${dummy}.*+0.00 ${rootname}NL.daospec | awk '{print $3}'`
	    if [ -n "$ewNU" -a -n "$ewNL" ]; then
     	      line=`grep ${dummy}.*+0.00 ${rootname}.daospec`
	      if [ -n "$line" ]; then
     	        echo $line | awk -v PIP="$ewNL" -v PLU="$ewNU" '{print $0"  "$3-PIP"  "$3-PLU}' >> $outfM04
	      fi
	    fi
          done
	fi
      fi
    rm -f ${rootname}NU.daospec ${rootname}NL.daospec
    rm -f ${rootname}NU.fits ${rootname}NL.fits
    rm -f ${rootname}NUC.fits ${rootname}NLC.fits
    rm -f ${rootname}NUR.fits ${rootname}NLR.fits
    rm -f ${path}/arith.cl 
    rm -f ${path}/login.cl
    rm -f ${path}/input.iraf
    fi
  fi
  ((index++))
done
cd $WORK_DIR 
echo -e "\nM04: module 04 analysed ${#list[@]} spectra. $errs spectra encountered some problems during the anlysis. $notd spectra of $errs do not have DAOSPEC output." 
echo M04: Files \*M04.daospec were created for spectrum that do not encounter errors. These files can be used for continuum error statistics.
if [ -s $out05 ]; then echo M04: Warnings and errors are reported in $out05.;fi 
echo M04: module 04 has ended its duty!

exit 0
      
    
    
   
     
